@@ -362,7 +362,7 @@ void ConfigDialogue::SetProperties()
   m_indentMaths->SetToolTip(
           _("Indent maths so all lines are in par with the first line that starts after the label."));
 
-  wxConfig *config = (wxConfig *) wxConfig::Get();
+  wxConfigBase *config = wxConfig::Get();
   wxString mp, mc, ib, mf;
 
   // The default values for all config items that will be used if there is no saved
@@ -673,6 +673,11 @@ wxPanel *ConfigDialogue::CreateStartupPanel()
   }
   m_wxStartupCommands = new wxTextCtrl(panel_wxMaximaStartup, -1, wxEmptyString, wxDefaultPosition, wxSize(200,250),
                                      wxTE_MULTILINE | wxHSCROLL);
+  #ifdef __WXMAC__
+    #if wxCHECK_VERSION(3, 1, 1)
+      m_wxStartupCommands->OSXDisableAllSmartSubstitutions(true);
+    #endif
+  #endif
   m_wxStartupCommands->SetValue(contents);
   vsizer_wxMaximaStartup->Add(m_wxStartupCommands, wxSizerFlags().Expand().Border(wxALL,5));
   wxStaticText *wxStartupFileLocation = new wxStaticText(panel_wxMaximaStartup, wxID_ANY,
@@ -713,6 +718,11 @@ wxPanel *ConfigDialogue::CreateStartupPanel()
   m_startupCommands = new wxTextCtrl(panel_maximaStartup, -1, wxEmptyString, wxDefaultPosition, wxSize(200,250),
                                      wxTE_MULTILINE | wxHSCROLL);
   m_startupCommands->SetValue(contents);
+  #ifdef __WXMAC__
+    #if wxCHECK_VERSION(3, 1, 1)
+      m_StartupCommands->OSXDisableAllSmartSubstitutions(true);
+    #endif
+  #endif
 
   vsizer_maximaStartup->Add(m_startupCommands, wxSizerFlags().Expand().Border(wxALL,5));
   wxStaticText *startupFileLocation = new wxStaticText(panel_maximaStartup, wxID_ANY,
@@ -1058,6 +1068,8 @@ wxPanel *ConfigDialogue::CreateStylePanel()
                   _("Highlight (dpart)"),
                   _("Maxima warnings"),
                   _("Text cell"),
+                  _("Heading 6"),
+                  _("Heading 5"),
                   _("Subsubsection cell"),
                   _("Subsection cell"),
                   _("Section cell"),
@@ -1163,7 +1175,7 @@ void ConfigDialogue::WriteSettings()
 {
   wxString search = wxT("maxima-htmldir");
   wxArrayString out;
-  wxConfig *config = (wxConfig *) wxConfig::Get();
+  wxConfigBase *config = wxConfig::Get();
   Configuration *configuration = m_configuration;
   configuration->SetAbortOnError(m_abortOnError->GetValue());
   configuration->RestartOnReEvaluation(m_restartOnReEvaluation->GetValue());
@@ -1266,7 +1278,7 @@ void ConfigDialogue::WriteSettings()
 
 void ConfigDialogue::OnMpBrowse(wxCommandEvent&  WXUNUSED(event))
 {
-  wxConfig *config = (wxConfig *) wxConfig::Get();
+  wxConfigBase *config = wxConfig::Get();
   wxString dd;
   config->Read(wxT("maxima"), &dd);
   wxString file = wxFileSelector(_("Select Maxima program"),
@@ -1340,6 +1352,8 @@ void ConfigDialogue::OnChangeFontFamily(wxCommandEvent &event)
   if (
     (tmp == &m_styleText)          ||
     (tmp == &m_styleTitle)         ||
+    (tmp == &m_styleHeading6)      ||
+    (tmp == &m_styleHeading5)      ||
     (tmp == &m_styleSubsubsection) ||
     (tmp == &m_styleSubsection)    ||
     (tmp == &m_styleSection))
@@ -1628,6 +1642,31 @@ void ConfigDialogue::ReadStyles(wxString file)
   m_styleCodeHighlightingEndOfLine.underlined = false;
   READ_STYLE(m_styleCodeHighlightingEndOfLine, "Style/CodeHighlighting/EndOfLine/")
 
+  // Heading 6
+  m_styleHeading6.color = wxT("black");
+  m_styleHeading6.bold = true;
+  m_styleHeading6.italic = false;
+  m_styleHeading6.underlined = false;
+  m_styleHeading6.font = m_styleDefault.font;
+  m_styleHeading6.fontSize = 14;
+  config->Read(wxT("Style/Heading6/fontsize"),
+               &m_styleHeading6.fontSize);
+  config->Read(wxT("Style/Heading6/fontname"),
+               &m_styleHeading6.font);
+  READ_STYLE(m_styleHeading6, "Style/Heading6/")
+
+  // Heading 5
+  m_styleHeading5.color = wxT("black");
+  m_styleHeading5.bold = true;
+  m_styleHeading5.italic = false;
+  m_styleHeading5.underlined = false;
+  m_styleHeading5.font = m_styleDefault.font;
+  m_styleHeading5.fontSize = 15;
+  config->Read(wxT("Style/Heading5/fontsize"),
+               &m_styleHeading5.fontSize);
+  config->Read(wxT("Style/Heading5/fontname"),
+               &m_styleHeading5.font);
+  READ_STYLE(m_styleHeading5, "Style/Heading5/")
   // Subsubsection
   m_styleSubsubsection.color = wxT("black");
   m_styleSubsubsection.bold = true;
@@ -1777,6 +1816,16 @@ void ConfigDialogue::WriteStyles(wxString file)
   WRITE_STYLE(m_styleCodeHighlightingOperator, "Style/CodeHighlighting/Operator/")
   WRITE_STYLE(m_styleCodeHighlightingEndOfLine, "Style/CodeHighlighting/EndOfLine/")
 
+  // Heading6
+  config->Write(wxT("Style/Heading6/fontname"), m_styleHeading6.font);
+  config->Write(wxT("Style/Heading6/fontsize"), m_styleHeading6.fontSize);
+  WRITE_STYLE(m_styleHeading6, "Style/Heading6/")
+
+  // Heading5
+  config->Write(wxT("Style/Heading5/fontname"), m_styleHeading5.font);
+  config->Write(wxT("Style/Heading5/fontsize"), m_styleHeading5.fontSize);
+  WRITE_STYLE(m_styleHeading5, "Style/Heading5/")
+
   // Subsubsection
   config->Write(wxT("Style/Subsubsection/fontname"), m_styleSubsubsection.font);
   config->Write(wxT("Style/Subsubsection/fontsize"), m_styleSubsubsection.fontSize);
@@ -1937,60 +1986,66 @@ style *ConfigDialogue::GetStylePointer()
       tmp = &m_styleText;
       break;
     case 15:
-      tmp = &m_styleSubsubsection;
+      tmp = &m_styleHeading6;
       break;
     case 16:
-      tmp = &m_styleSubsection;
+      tmp = &m_styleHeading5;
       break;
     case 17:
-      tmp = &m_styleSection;
+      tmp = &m_styleSubsubsection;
       break;
     case 18:
-      tmp = &m_styleTitle;
+      tmp = &m_styleSubsection;
       break;
     case 19:
-      tmp = &m_styleTextBackground;
+      tmp = &m_styleSection;
       break;
     case 20:
-      tmp = &m_styleBackground;
+      tmp = &m_styleTitle;
       break;
     case 21:
-      tmp = &m_styleCellBracket;
+      tmp = &m_styleTextBackground;
       break;
     case 22:
-      tmp = &m_styleActiveCellBracket;
+      tmp = &m_styleBackground;
       break;
     case 23:
-      tmp = &m_styleCursor;
+      tmp = &m_styleCellBracket;
       break;
     case 24:
-      tmp = &m_styleSelection;
+      tmp = &m_styleActiveCellBracket;
       break;
     case 25:
-      tmp = &m_styleEqualsSelection;
+      tmp = &m_styleCursor;
       break;
     case 26:
-      tmp = &m_styleOutdated;
+      tmp = &m_styleSelection;
       break;
     case 27:
-      tmp = &m_styleCodeHighlightingVariable;
+      tmp = &m_styleEqualsSelection;
       break;
     case 28:
-      tmp = &m_styleCodeHighlightingFunction;
+      tmp = &m_styleOutdated;
       break;
     case 29:
-      tmp = &m_styleCodeHighlightingComment;
+      tmp = &m_styleCodeHighlightingVariable;
       break;
     case 30:
-      tmp = &m_styleCodeHighlightingNumber;
+      tmp = &m_styleCodeHighlightingFunction;
       break;
     case 31:
-      tmp = &m_styleCodeHighlightingString;
+      tmp = &m_styleCodeHighlightingComment;
       break;
     case 32:
-      tmp = &m_styleCodeHighlightingOperator;
+      tmp = &m_styleCodeHighlightingNumber;
       break;
     case 33:
+      tmp = &m_styleCodeHighlightingString;
+      break;
+    case 34:
+      tmp = &m_styleCodeHighlightingOperator;
+      break;
+    case 35:
       tmp = &m_styleCodeHighlightingEndOfLine;
       break;
     default:
@@ -2013,8 +2068,9 @@ void ConfigDialogue::UpdateExample()
     color = m_styleInput.color;
 
   int fontsize = m_fontSize;
-  if (tmp == &m_styleText || tmp == &m_styleSubsubsection || tmp == &m_styleSubsection
-      || tmp == &m_styleSection || tmp == &m_styleTitle)
+  if (tmp == &m_styleText || tmp == &m_styleHeading5 || tmp == &m_styleHeading6 ||
+      tmp == &m_styleSubsubsection || tmp == &m_styleSubsection ||
+      tmp == &m_styleSection || tmp == &m_styleTitle)
   {
     fontsize = tmp->fontSize;
     font = tmp->font;
