@@ -336,7 +336,7 @@ public:
       )
       return m_clientWidth;
     else
-      return (const int) (double(m_defaultFontSize) * LineWidth_em() * m_zoomFactor);
+      return (int) (double(m_defaultFontSize) * LineWidth_em() * m_zoomFactor);
   }
 
   int GetDefaultFontSize()
@@ -378,7 +378,7 @@ public:
   void IndentMaths(bool indent){wxConfig::Get()->Write(wxT("indentMaths"), m_indentMaths=indent);}
   int GetFontSize(int st)
   {
-    if (st == TS_TEXT || st == TS_SUBSUBSECTION || st == TS_SUBSECTION || st == TS_SECTION || st == TS_TITLE)
+    if (st == TS_TEXT || st == TS_HEADING5 || st == TS_HEADING6 || st == TS_SUBSUBSECTION || st == TS_SUBSECTION || st == TS_SECTION || st == TS_TITLE)
       return m_styles[st].fontSize;
     return 0;
   }
