@@ -117,6 +117,18 @@ GroupCell::GroupCell(Configuration **config, int groupType, CellPointers *cellPo
       editor->SetType(MC_TYPE_SUBSUBSECTION);
       AppendInput(editor);
       break;
+    case GC_TYPE_HEADING5:
+      m_inputLabel->SetType(MC_TYPE_HEADING5);
+      editor = new EditorCell(this, m_configuration, m_cellPointers);
+      editor->SetType(MC_TYPE_HEADING5);
+      AppendInput(editor);
+      break;
+    case GC_TYPE_HEADING6:
+      m_inputLabel->SetType(MC_TYPE_HEADING6);
+      editor = new EditorCell(this, m_configuration, m_cellPointers);
+      editor->SetType(MC_TYPE_HEADING6);
+      AppendInput(editor);
+      break;
     case GC_TYPE_IMAGE:
       m_inputLabel->SetType(MC_TYPE_TEXT);
       editor = new EditorCell(this, m_configuration, m_cellPointers);
@@ -204,6 +216,24 @@ void GroupCell::SetCellStyle(int style)
       GetEditable()->SetType(MC_TYPE_SUBSUBSECTION);
     RemoveOutput();
     break;
+  case GC_TYPE_HEADING5:
+    m_inputLabel -> SetValue(wxEmptyString);
+    m_groupType = GC_TYPE_HEADING5;
+    if(m_inputLabel != NULL)
+      m_inputLabel->SetType(MC_TYPE_HEADING5);
+    if(GetEditable() != NULL)
+      GetEditable()->SetType(MC_TYPE_HEADING5);
+    RemoveOutput();
+    break;
+  case GC_TYPE_HEADING6:
+    m_inputLabel -> SetValue(wxEmptyString);
+    m_groupType = GC_TYPE_HEADING6;
+    if(m_inputLabel != NULL)
+      m_inputLabel->SetType(MC_TYPE_HEADING6);
+    if(GetEditable() != NULL)
+      GetEditable()->SetType(MC_TYPE_HEADING6);
+    RemoveOutput();
+    break;
   }
   if(GetEditable() != NULL)
     GetEditable()->StyleText();
@@ -330,6 +360,16 @@ wxString GroupCell::ToWXM(bool wxm)
       retval += GetEditable()->ToString() + wxT("\n");
       retval += wxT("   [wxMaxima: subsubsect end   ] */\n");
       break;
+    case GC_TYPE_HEADING5:
+      retval += wxT("/* [wxMaxima: heading5 start ]\n");
+      retval += GetEditable()->ToString() + wxT("\n");
+      retval += wxT("   [wxMaxima: heading5 end   ] */\n");
+      break;
+    case GC_TYPE_HEADING6:
+      retval += wxT("/* [wxMaxima: heading6 start ]\n");
+      retval += GetEditable()->ToString() + wxT("\n");
+      retval += wxT("   [wxMaxima: heading6 end   ] */\n");
+      break;
     case GC_TYPE_TITLE:
       retval += wxT("/* [wxMaxima: title   start ]\n");
       retval += GetEditable()->ToString() + wxT("\n");
@@ -542,18 +582,18 @@ void GroupCell::AppendOutput(MathCell *cell)
 
 void GroupCell::Recalculate()
 {
-  int d_fontsize = (*m_configuration)->GetDefaultFontSize();
-  int m_fontsize = (*m_configuration)->GetMathFontSize();
+  int fontsize = (*m_configuration)->GetDefaultFontSize();
+  
+  m_fontSize = fontsize;
+  m_mathFontSize = (*m_configuration)->GetMathFontSize();
 
-  m_fontSize = d_fontsize;
-  m_mathFontSize = m_fontsize;
-
-  RecalculateWidths(d_fontsize);
-  RecalculateHeight(d_fontsize);
+  RecalculateWidths(fontsize);
+  RecalculateHeight(fontsize);
 }
 
 void GroupCell::RecalculateWidths(int fontsize)
 {
+  MathCell::RecalculateWidths(fontsize);
   Configuration *configuration = (*m_configuration);
   if (m_width == -1 || m_height == -1 || configuration->ForceUpdate())
   {
@@ -573,10 +613,7 @@ void GroupCell::RecalculateWidths(int fontsize)
       m_inputLabel->RecalculateWidthsList(fontsize);
       
       // recalculate the position of input in ReEvaluateSelection!
-      if (m_inputLabel->m_next != NULL)
-      {
-        m_inputLabel->m_next->m_currentPoint.x = m_currentPoint.x + m_inputLabel->GetWidth();
-      }
+      // m_inputLabel->m_currentPoint.x = m_currentPoint.x;
     }
     
     if (m_output == NULL || m_hide)
@@ -703,8 +740,11 @@ void GroupCell::RecalculateHeightInput(int fontsize)
   m_outputRect.x = m_currentPoint.x;
   m_outputRect.y = m_currentPoint.y;
   if (m_output) m_outputRect.y -= m_output->GetMaxCenter();
-  m_outputRect.width = 0;
-  m_outputRect.height = 0;
+  else
+  {
+    m_outputRect.width = 0;
+    m_outputRect.height = 0;
+  }
   if ((configuration->ShowCodeCells()) ||
       (m_groupType != GC_TYPE_CODE))
   {
@@ -733,6 +773,7 @@ void GroupCell::RecalculateHeightOutput(int WXUNUSED(fontsize))
 
 void GroupCell::RecalculateHeight(int fontsize)
 {
+  MathCell::RecalculateHeight(fontsize);
   Configuration *configuration = (*m_configuration);
 
   if (m_width < 0 || m_height < 0 || m_currentPoint.x < 0 || m_currentPoint.y < 0 ||
@@ -770,12 +811,13 @@ void GroupCell::RecalculateHeight(int fontsize)
   // bracket in.  m_appendedCells = NULL;
 
   if (m_inputLabel)
-    m_inputLabel->m_currentPoint = m_currentPoint;
+    m_inputLabel->SetCurrentPoint(m_currentPoint);
   if (GetEditable())
   {
-    GetEditable()->m_currentPoint = m_currentPoint;
-    if(m_inputLabel != NULL)
-      GetEditable()->m_currentPoint.x += m_inputLabel->GetWidth();
+    if(m_inputLabel == NULL)
+      GetEditable()->SetCurrentPoint(m_currentPoint);
+    else
+      GetEditable()->SetCurrentPoint(wxPoint(m_currentPoint.x + m_inputLabel->GetWidth(), m_currentPoint.y));
   }
 }
 
@@ -843,20 +885,16 @@ void GroupCell::RecalculateAppended()
   m_appendedCells = NULL;
 }
 
-void GroupCell::Draw(wxPoint point, int fontsize)
+void GroupCell::Draw(wxPoint point)
 {
+  MathCell::Draw(point);
   if (DrawThisCell(point))
   {
-    
-    MathCell::Draw(point, fontsize);
+    if (m_width == -1 || m_height == -1)
+      return;
     
     Configuration *configuration = (*m_configuration);
     wxDC *dc = configuration->GetDC();
-    if (m_width == -1 || m_height == -1)
-    {
-      RecalculateWidths(fontsize);
-      RecalculateHeight(fontsize);
-    }
     // draw a thick line for 'page break'
     // and return
     if (m_groupType == GC_TYPE_PAGEBREAK)
@@ -866,7 +904,6 @@ void GroupCell::Draw(wxPoint point, int fontsize)
       wxPen pen(configuration->GetColor(TS_CURSOR), 1, wxPENSTYLE_DOT);
       dc->SetPen(pen);
       dc->DrawLine(0, y, (*m_configuration)->GetCanvasSize().GetWidth(), y);
-      MathCell::Draw(point, fontsize);
       return;
     }
 
@@ -877,7 +914,6 @@ void GroupCell::Draw(wxPoint point, int fontsize)
 
     if(configuration->GetIndent() < rect.GetRight())
     {
-       
       if(rect.GetLeft() <= configuration->GetCellBracketWidth())
         rect.SetLeft(configuration->GetIndent());
       
@@ -909,7 +945,20 @@ void GroupCell::Draw(wxPoint point, int fontsize)
           (m_groupType != GC_TYPE_CODE))
       {
         configuration->Outdated(false);
-        m_inputLabel->DrawList(in, fontsize);
+        int labelWidth = Scale_Px(configuration->GetLabelWidth());
+        if(m_inputLabel)
+        {
+          m_inputLabel->Draw(in);
+          labelWidth = MAX(labelWidth, m_inputLabel->GetWidth());
+        }
+        EditorCell *input = GetInput();
+        if(input)
+          input->Draw(
+            wxPoint(
+              in.x + labelWidth,
+              in.y
+              )
+            );
         if (m_groupType == GC_TYPE_CODE && m_inputLabel->m_next)
           configuration->Outdated((dynamic_cast<EditorCell *>(m_inputLabel->m_next))->ContainsChanges());
       }
@@ -920,66 +969,22 @@ void GroupCell::Draw(wxPoint point, int fontsize)
         int drop = tmp->GetMaxDrop();
         if ((configuration->ShowCodeCells()) ||
             (m_groupType != GC_TYPE_CODE))
-        {
           in.y += m_inputLabel->GetMaxDrop();
-        }
+
         in.y += m_output->GetMaxCenter();
         m_outputRect.y = in.y - m_output->GetMaxCenter();
         m_outputRect.x = in.x;
 
         while (tmp != NULL)
-        {
-          if (tmp->BreakLineHere())
-          {
-            if (tmp->m_bigSkip)
-              in.y += MC_LINE_SKIP;
-          
-            if (tmp->m_previousToDraw != NULL &&
-                tmp->GetStyle() == TS_LABEL)
-              in.y += configuration->GetInterEquationSkip();
-          }
-        
-          tmp->m_currentPoint = in;
+        {         
+          tmp->SetCurrentPoint(in);
 
-          if(
-              (tmp->GetStyle() != TS_LABEL) &&
-              (tmp->GetStyle() != TS_USERLABEL) &&
-              (tmp->GetStyle() != TS_MAIN_PROMPT) &&
-              (tmp->GetStyle() != TS_OTHER_PROMPT) &&
-              (tmp->m_previousToDraw == NULL) &&
-              configuration->IndentMaths()
-            )
-            in.x += Scale_Px(configuration->GetLabelWidth()) + MC_TEXT_PADDING;
-          
-          if (!tmp->m_isBroken)
-          {
-            if (tmp->DrawThisCell(in))
-              tmp->Draw(in, MAX(tmp->IsMath() ? m_mathFontSize : m_fontSize, MC_MIN_SIZE));
-            if (tmp->m_nextToDraw != NULL)
-            {
-              if (tmp->m_nextToDraw->BreakLineHere())
-              {
-                in.x = point.x;
-                if(
-                  (tmp->m_nextToDraw->GetStyle() != TS_LABEL) &&
-                  (tmp->m_nextToDraw->GetStyle() != TS_USERLABEL) &&
-                  (tmp->m_nextToDraw->GetStyle() != TS_MAIN_PROMPT) &&
-                  (tmp->m_nextToDraw->GetStyle() != TS_OTHER_PROMPT) &&
-                  configuration->IndentMaths()
-                  )
-                  in.x += Scale_Px(configuration->GetLabelWidth()) + MC_TEXT_PADDING;
-                in.y += drop + tmp->m_nextToDraw->GetMaxCenter();
-                drop = tmp->m_nextToDraw->GetMaxDrop();
-              }
-              else
-                in.x += (tmp->GetWidth());
-            }
-
-          }
-          else
-          {
-            if (tmp->m_nextToDraw != NULL && tmp->m_nextToDraw->BreakLineHere())
+          tmp->Draw(in);
+          if ((tmp->m_nextToDraw != NULL) && (tmp->m_nextToDraw->BreakLineHere()))
             {
+              if (tmp->m_nextToDraw->m_bigSkip)
+                in.y += MC_LINE_SKIP;
+ 
               in.x = point.x;
               if(
                 (tmp->m_nextToDraw->GetStyle() != TS_LABEL) &&
@@ -989,10 +994,13 @@ void GroupCell::Draw(wxPoint point, int fontsize)
                 configuration->IndentMaths()
                 )
                 in.x += Scale_Px(configuration->GetLabelWidth()) + MC_TEXT_PADDING;
+              
               in.y += drop + tmp->m_nextToDraw->GetMaxCenter();
               drop = tmp->m_nextToDraw->GetMaxDrop();
             }
-          }
+          else
+            in.x += tmp->GetWidth();
+
           tmp = tmp->m_nextToDraw;
         }
       }
@@ -1362,6 +1370,12 @@ wxString GroupCell::ToTeX(wxString imgDir, wxString filename, int *imgCounter)
           case TS_SUBSUBSECTION:
             str = wxT("\n\\subsubsection{") + str + wxT("}\n");
             break;
+          case TS_HEADING5:
+            str = wxT("\n\\paragraph{") + str + wxT("}\n");
+            break;
+          case TS_HEADING6:
+            str = wxT("\n\\subparagraph{") + str + wxT("}\n");
+            break;
           default:
             if (str.StartsWith(wxT("TeX:")))
             {
@@ -1571,6 +1585,12 @@ wxString GroupCell::ToXML()
       // as subsections on old wxMaxima installations.
       str += wxT(" type=\"subsection\" sectioning_level=\"4\"");
       break;
+    case GC_TYPE_HEADING5:
+      str += wxT(" type=\"subsection\" sectioning_level=\"5\"");
+      break;
+    case GC_TYPE_HEADING6:
+      str += wxT(" type=\"subsection\" sectioning_level=\"6\"");
+      break;
     case GC_TYPE_PAGEBREAK:
     {
       str += wxT(" type=\"pagebreak\"/>");
@@ -1621,6 +1641,8 @@ wxString GroupCell::ToXML()
     case GC_TYPE_SECTION:
     case GC_TYPE_SUBSECTION:
     case GC_TYPE_SUBSUBSECTION:
+    case GC_TYPE_HEADING5:
+    case GC_TYPE_HEADING6:
       if (input)
         str += input->ListToXML();
       if (m_hiddenTree)
@@ -1824,6 +1846,8 @@ EditorCell *GroupCell::GetEditable()
     case GC_TYPE_SECTION:
     case GC_TYPE_SUBSECTION:
     case GC_TYPE_SUBSUBSECTION:
+    case GC_TYPE_HEADING5:
+    case GC_TYPE_HEADING6:
     default:
       return GetInput();
   }
@@ -1845,7 +1869,7 @@ void GroupCell::BreakLines(MathCell *cell, int fullWidth)
   {
     tmp->ResetData();
     tmp->BreakLine(false);
-    if (!tmp->m_isBroken)
+    if (!tmp->m_isBrokenIntoLines)
     {
       if (tmp->BreakLineHere() || (currentWidth + tmp->GetWidth() >= fullWidth))
       {
@@ -1899,7 +1923,9 @@ void GroupCell::BreakUpCells(MathCell *cell, int WXUNUSED(fontsize), int clientW
     
   while (tmp != NULL && !m_hide)
   {
-    if (tmp->GetWidth() > clientWidth)
+    if (tmp->GetWidth() +
+        (*m_configuration)->GetIndent() +
+        Scale_Px((*m_configuration)->GetLabelWidth()) > clientWidth)
     {
       if (tmp->BreakUp())
         lineHeightsChanged = true;
@@ -1909,14 +1935,10 @@ void GroupCell::BreakUpCells(MathCell *cell, int WXUNUSED(fontsize), int clientW
 
   if(lineHeightsChanged)
   {
-    if(m_inputLabel != NULL)
-    {
-      m_inputLabel->RecalculateList((*m_configuration)->GetDefaultFontSize());
-    }
     if(m_output != NULL)
     {
       m_output->ResetSizeList();
-      m_output->RecalculateList(m_output->IsMath() ? m_mathFontSize : m_fontSize);
+      m_output->RecalculateList(m_mathFontSize);
     }
     ResetData();
   }
@@ -1927,11 +1949,11 @@ void GroupCell::UnBreakUpCells()
   MathCell *tmp = m_output;
   while (tmp != NULL)
   {
-    if (tmp->m_isBroken)
+    if (tmp->m_isBrokenIntoLines)
     {
       tmp->Unbreak();
     }
-    tmp = tmp->m_next;
+    tmp = tmp->m_nextToDraw;
   }
 }
 
@@ -2145,7 +2167,15 @@ bool GroupCell::IsLesserGCType(int comparedTo)
     case GC_TYPE_PAGEBREAK:
     case GC_TYPE_IMAGE:
       return (comparedTo == GC_TYPE_TITLE) || (comparedTo == GC_TYPE_SECTION) ||
-             (comparedTo == GC_TYPE_SUBSECTION) || (comparedTo == GC_TYPE_SUBSUBSECTION);
+             (comparedTo == GC_TYPE_SUBSECTION) || (comparedTo == GC_TYPE_SUBSUBSECTION) ||
+        (comparedTo == GC_TYPE_HEADING5) || (comparedTo == GC_TYPE_HEADING6);
+    case GC_TYPE_HEADING6:
+      return (comparedTo == GC_TYPE_TITLE) || (comparedTo == GC_TYPE_SECTION)
+             || (comparedTo == GC_TYPE_SUBSECTION) || (comparedTo == GC_TYPE_SUBSUBSECTION) ||
+        (comparedTo == GC_TYPE_HEADING5);
+    case GC_TYPE_HEADING5:
+      return (comparedTo == GC_TYPE_TITLE) || (comparedTo == GC_TYPE_SECTION)
+             || (comparedTo == GC_TYPE_SUBSECTION)|| (comparedTo == GC_TYPE_SUBSUBSECTION);
     case GC_TYPE_SUBSUBSECTION:
       return (comparedTo == GC_TYPE_TITLE) || (comparedTo == GC_TYPE_SECTION)
              || (comparedTo == GC_TYPE_SUBSECTION);
@@ -2160,7 +2190,7 @@ bool GroupCell::IsLesserGCType(int comparedTo)
   }
 }
 
-void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &image)
+void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &heading5, int &heading6, int &image)
 {
   GroupCell *tmp = this;
 
@@ -2169,11 +2199,11 @@ void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &i
     switch (tmp->m_groupType)
     {
       case GC_TYPE_TITLE:
-        section = subsection = subsubsection = 0;
+        section = subsection = subsubsection = heading5 = heading6 = 0;
         break;
       case GC_TYPE_SECTION:
         section++;
-        subsection = subsubsection = 0;
+        subsection = subsubsection = heading5 = heading6 = 0;
         {
           wxString num = wxT("  ");
           num << section << wxT(" ");
@@ -2182,7 +2212,7 @@ void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &i
         }
         break;
       case GC_TYPE_SUBSECTION:
-        subsubsection = 0;
+        subsubsection = heading5 = heading6 = 0;
         subsection++;
         {
           wxString num = wxT("  ");
@@ -2192,6 +2222,7 @@ void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &i
         }
         break;
       case GC_TYPE_SUBSUBSECTION:
+        heading5 = heading6 = 0;
         subsubsection++;
         {
           wxString num = wxT("  ");
@@ -2200,6 +2231,27 @@ void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &i
           tmp->m_inputLabel->SetStyle(TS_SUBSUBSECTION);
         }
         break;
+      case GC_TYPE_HEADING5:
+        heading5++;
+        heading6 = 0;
+        {
+          wxString num = wxT("  ");
+          num << section << wxT(".") << subsection << wxT(".") << subsubsection << wxT(".")
+              << heading5 << wxT(" ");
+          tmp->m_inputLabel->SetValue(num);
+          tmp->m_inputLabel->SetStyle(TS_HEADING5);
+        }
+        break;
+      case GC_TYPE_HEADING6:
+        heading6++;
+        {
+          wxString num = wxT("  ");
+          num << section << wxT(".") << subsection << wxT(".") << subsubsection << wxT(".")
+              << heading5 << wxT(".") << heading6 << wxT(" ");
+          tmp->m_inputLabel->SetValue(num);
+          tmp->m_inputLabel->SetStyle(TS_HEADING6);
+        }
+        break;
       case GC_TYPE_IMAGE:
         image++;
         {
@@ -2212,7 +2264,7 @@ void GroupCell::Number(int &section, int &subsection, int &subsubsection, int &i
     }
 
     if (IsFoldable() && tmp->m_hiddenTree)
-      tmp->m_hiddenTree->Number(section, subsection, subsubsection, image);
+      tmp->m_hiddenTree->Number(section, subsection, subsubsection, heading5, heading6, image);
 
     tmp = dynamic_cast<GroupCell *>(tmp->m_next);
   }
