@@ -71,15 +71,15 @@ MathCell::MathCell(MathCell *group, Configuration **config)
   m_forceBreakLine = false;
   m_bigSkip = true;
   m_isHidden = false;
-  m_isBroken = false;
+  m_isBrokenIntoLines = false;
   m_highlight = false;
   m_type = MC_TYPE_DEFAULT;
   m_textStyle = TS_VARIABLE;
   m_SuppressMultiplicationDot = false;
   m_imageBorderWidth = 0;
-  m_currentPoint.x = -1;
-  m_currentPoint.y = -1;
+  SetCurrentPoint(wxPoint(-1, -1));
   m_toolTip = (*m_configuration)->GetDefaultMathCellToolTip();
+  m_fontSize = (*m_configuration)->GetMathFontSize();
 }
 
 MathCell::~MathCell()
@@ -129,6 +129,12 @@ void MathCell::SetType(int type)
     case MC_TYPE_SUBSUBSECTION:
       m_textStyle = TS_SUBSUBSECTION;
       break;
+    case MC_TYPE_HEADING5:
+      m_textStyle = TS_HEADING5;
+      break;
+    case MC_TYPE_HEADING6:
+      m_textStyle = TS_HEADING6;
+      break;
     case MC_TYPE_SUBSECTION:
       m_textStyle = TS_SUBSECTION;
       break;
@@ -249,10 +255,10 @@ int MathCell::GetMaxDrop()
     MathCell *tmp = this;
     while (tmp != NULL)
     {
-      m_maxDrop = MAX(m_maxDrop, tmp->m_isBroken ? 0 : (tmp->m_height - tmp->m_center));
+      m_maxDrop = MAX(m_maxDrop, tmp->m_isBrokenIntoLines ? 0 : (tmp->m_height - tmp->m_center));
       if (tmp->m_nextToDraw == NULL)
         break;
-      if (tmp->m_nextToDraw->m_breakLine && !tmp->m_nextToDraw->m_isBroken)
+      if (tmp->m_nextToDraw->m_breakLine && !tmp->m_nextToDraw->m_isBrokenIntoLines)
         break;
       tmp = tmp->m_nextToDraw;
     }
@@ -308,7 +314,7 @@ int MathCell::GetLineWidth()
       tmp = tmp->m_nextToDraw;
       if(tmp != NULL)
       {
-        if(tmp->m_isBroken || tmp->m_breakLine || (tmp->m_type == MC_TYPE_MAIN_PROMPT))
+        if(tmp->m_isBrokenIntoLines || tmp->m_breakLine || (tmp->m_type == MC_TYPE_MAIN_PROMPT))
           break;
       }
     }
@@ -321,9 +327,9 @@ int MathCell::GetLineWidth()
  To make this work each derived class must draw the content of the cell
  and then call MathCall::Draw(...).
  */
-void MathCell::Draw(wxPoint point, int WXUNUSED(fontsize))
+void MathCell::Draw(wxPoint point)
 {
-  m_currentPoint = point;
+  SetCurrentPoint(point);
 
   // Tell the screen reader that this cell's contents might have changed.
 
@@ -333,12 +339,12 @@ void MathCell::Draw(wxPoint point, int WXUNUSED(fontsize))
 #endif
 }
 
-void MathCell::DrawList(wxPoint point, int fontsize)
+void MathCell::DrawList(wxPoint point)
 {
   MathCell *tmp = this;
   while (tmp != NULL)
   {
-    tmp->Draw(point, fontsize);
+    tmp->Draw(point);
     point.x += tmp->m_width;
     wxASSERT(tmp != tmp->m_nextToDraw);
     tmp = tmp->m_nextToDraw;
@@ -353,7 +359,7 @@ void MathCell::RecalculateList(int fontsize)
   {
     tmp->RecalculateWidths(fontsize);
     tmp->RecalculateHeight(fontsize);
-    tmp = tmp->m_next;
+    tmp = tmp->m_nextToDraw;
   }
 }
 
@@ -398,23 +404,39 @@ void MathCell::RecalculateWidthsList(int fontsize)
   }
 }
 
-void MathCell::RecalculateWidths(int WXUNUSED(fontsize))
+void MathCell::RecalculateWidths(int fontsize)
 {
   ResetData();
+  m_fontSize = fontsize;
 }
 
 /*! Is this cell currently visible in the window?.
  */
 bool MathCell::DrawThisCell(wxPoint point)
 {
+  // If the cell isn't on the worksheet we don't draw it.
   if((point.x < 0) || (point.y < 0))
     return false;
 
+    if(!m_clipToDrawRegion)
+      return true;
+  
+  SetCurrentPoint(point);
+
+  // If a cell is broken into lines its individual parts are displayed but
+  // not the cell itself (example: Denominator and Numerator are displayed
+  // but not the horizontal line with denominator above and numerator below.
+  if(m_isBrokenIntoLines)
+    return false;
+
+  // We need to redraw group cell brackets on cursor movements.
+  // TODO: Is this the right method to make this happen?
+  if((m_type != MC_TYPE_GROUP) && (!InUpdateRegion()))
+    return false;
+    
   Configuration *configuration = (*m_configuration);
   int top = configuration->GetTop();
   int bottom = configuration->GetBottom();
-  if (top == -1 || bottom == -1)
-    return true;
   if (point.y - GetMaxCenter() > bottom || point.y + GetMaxDrop() < top)
     return false;
   return true;
@@ -428,18 +450,17 @@ bool MathCell::DrawThisCell(wxPoint point)
  */
 wxRect MathCell::GetRect(bool all)
 {
-  if (m_isBroken)
-    return wxRect(-1, -1, 0, 0);
   if (all)
     return wxRect(m_currentPoint.x, m_currentPoint.y - GetMaxCenter(),
                   GetLineWidth(), GetMaxHeight());
-  return wxRect(m_currentPoint.x, m_currentPoint.y - m_center,
-                m_width, m_height);
+  else
+    return wxRect(m_currentPoint.x, m_currentPoint.y - m_center,
+                  m_width, m_height);
 }
 
 bool MathCell::InUpdateRegion(const wxRect &rect)
 {
-  if (m_clipToDrawRegion) return true;
+  if (!m_clipToDrawRegion) return true;
   if (rect.GetLeft() > m_updateRegion.GetRight()) return false;
   if (rect.GetRight() < m_updateRegion.GetLeft()) return false;
   if (rect.GetBottom() < m_updateRegion.GetTop()) return false;
@@ -961,7 +982,7 @@ void MathCell::SelectInner(wxRect &rect, MathCell **first, MathCell **last)
 
 bool MathCell::BreakLineHere()
 {
-  return (((!m_isBroken) && m_breakLine) || m_forceBreakLine);
+  return (((!m_isBrokenIntoLines) && m_breakLine) || m_forceBreakLine);
 }
 
 bool MathCell::ContainsRect(const wxRect &sm, bool all)
@@ -1013,7 +1034,7 @@ MathCell *MathCell::last()
 void MathCell::Unbreak()
 {
   ResetData();
-  m_isBroken = false;
+  m_isBrokenIntoLines = false;
   m_nextToDraw = m_next;
   if (m_nextToDraw != NULL)
     m_nextToDraw->m_previousToDraw = this;
@@ -1324,6 +1345,7 @@ wxAccStatus MathCell::GetRole (int WXUNUSED(childId), wxAccRole *role)
 
 MathCell::CellPointers::CellPointers(wxScrolledCanvas *mathCtrl)
 {
+  m_cellToScrollTo = NULL;
   m_wxmxImgCounter = 0;
   m_mathCtrl = mathCtrl;
   m_cellMouseSelectionStartedIn = NULL;
@@ -1362,6 +1384,10 @@ bool MathCell::CellPointers::ErrorList::Contains(MathCell *cell)
 void MathCell::MarkAsDeleted()
 {
   // Delete all pointers to this cell
+  if(this == m_cellPointers->CellToScrollTo())
+  {
+      m_cellPointers->m_scrollToCell = false;
+  }
   if(this == m_cellPointers->m_workingGroup)
     m_cellPointers->m_workingGroup = NULL;
   if(this == m_cellPointers->m_lastWorkingGroup)
