@@ -192,7 +192,7 @@ void Svgout::BreakLines()
 
   while (tmp != NULL)
   {
-    if (!tmp->m_isBroken)
+    if (!tmp->m_isBrokenIntoLines)
     {
       tmp->BreakLine(false);
       tmp->ResetData();
@@ -220,7 +220,7 @@ void Svgout::GetMaxPoint(int *width, int *height)
   bool firstCell = true;
   while (tmp != NULL)
   {
-    if (!tmp->m_isBroken)
+    if (!tmp->m_isBrokenIntoLines)
     {
       if (tmp->BreakLineHere() || firstCell)
       {
@@ -261,9 +261,9 @@ void Svgout::Draw()
 
     while (tmp != NULL)
     {
-      if (!tmp->m_isBroken)
+      if (!tmp->m_isBrokenIntoLines)
       {
-        tmp->Draw(point, tmp->IsMath() ? mfontsize : fontsize);
+        tmp->Draw(point);
         if ((tmp->m_next != NULL) && (tmp->m_next->BreakLineHere()))
         {
           point.x = 0;
@@ -328,6 +328,7 @@ Svgout::SVGDataObject *Svgout::GetDataObject()
 
 bool Svgout::ToClipboard()
 {
+  wxASSERT_MSG(!wxTheClipboard->IsOpened(),_("Bug: The clipboard is already opened"));
   if (wxTheClipboard->Open())
   {
     bool res = wxTheClipboard->SetData(GetDataObject());
