@@ -62,7 +62,7 @@ public:
 
   void RecalculateWidths(int fontsize);
 
-  void Draw(wxPoint point, int fontsize);
+  virtual void Draw(wxPoint point);
 
   bool BreakUp();
 
