@@ -64,6 +64,9 @@ void TextCell::SetStyle(int style)
     m_displayedText = wxT("\x0393");
   if ((m_text == wxT("psi")) && (m_textStyle == TS_FUNCTION))
     m_displayedText = wxT("\x03A8");
+  if((style == TS_LABEL) || (style == TS_USERLABEL)||
+     (style == TS_MAIN_PROMPT) || (style == TS_OTHER_PROMPT))
+    m_forceBreakLine = true;
 }
 
 void TextCell::SetValue(const wxString &text)
@@ -302,6 +305,7 @@ MathCell *TextCell::Copy()
 
 void TextCell::RecalculateWidths(int fontsize)
 {
+  MathCell::RecalculateWidths(fontsize);
   Configuration *configuration = (*m_configuration);
 
   bool recalculateNeeded = false;
@@ -368,10 +372,14 @@ void TextCell::RecalculateWidths(int fontsize)
       }
 
       wxFont font = dc->GetFont();
-      int fontsize1 = Scale_Px(configuration->GetDefaultFontSize());
+      double fontsize1 = Scale_Px(configuration->GetDefaultFontSize());
       if(fontsize1 < 4)
         fontsize1 = 4;
+#if wxCHECK_VERSION(3, 1, 2)
+      font.SetFractionalPointSize(fontsize1);
+#else
       font.SetPointSize(fontsize1);
+#endif
       dc->SetFont(font);
 
       
@@ -386,7 +394,11 @@ void TextCell::RecalculateWidths(int fontsize)
       font = dc->GetFont();
       do
       {
+#if wxCHECK_VERSION(3, 1, 2)
+        font.SetFractionalPointSize(Scale_Px(--m_fontSizeLabel));
+#else
         font.SetPointSize(Scale_Px(--m_fontSizeLabel));
+#endif
         dc->SetFont(font);
         dc->GetTextExtent(text, &labelWidth, &labelHeight);
       } while ((labelWidth >= m_width) && (m_fontSizeLabel > 2));
@@ -434,21 +446,21 @@ void TextCell::RecalculateWidths(int fontsize)
   ResetData();
 }
 
-void TextCell::Draw(wxPoint point, int fontsize)
+void TextCell::Draw(wxPoint point)
 {
   if (DrawThisCell(point) && !m_isHidden)
   {
     
     Configuration *configuration = (*m_configuration);
-    MathCell::Draw(point, fontsize);
+    MathCell::Draw(point);
     wxDC *dc = configuration->GetDC();
     
-    if (m_width == -1 || m_height == -1 || fontsize != m_lastCalculationFontSize)
-      RecalculateWidths(fontsize);
+    if (m_width == -1 || m_height == -1 || m_fontSize != m_lastCalculationFontSize)
+      RecalculateWidths(m_fontSize);
     
     if (InUpdateRegion())
     {
-      SetFont(fontsize);
+      SetFont(m_fontSize);
       SetForeground();
       /// Labels and prompts have special fontsize
       if ((m_textStyle == TS_LABEL) || (m_textStyle == TS_USERLABEL) || (m_textStyle == TS_MAIN_PROMPT))
@@ -522,7 +534,11 @@ void TextCell::Draw(wxPoint point, int fontsize)
 void TextCell::SetFontSizeForLabel(wxDC *dc)
 {
   wxFont font(dc->GetFont());
+#if wxCHECK_VERSION(3, 1, 2)
+  font.SetFractionalPointSize(Scale_Px(m_fontSizeLabel));
+#else
   font.SetPointSize(Scale_Px(m_fontSizeLabel));
+#endif
   dc->SetFont(font);
 }
 
@@ -535,7 +551,9 @@ void TextCell::SetFont(int fontsize)
   if ((m_textStyle == TS_TITLE) ||
       (m_textStyle == TS_SECTION) ||
       (m_textStyle == TS_SUBSECTION) ||
-      (m_textStyle == TS_SUBSUBSECTION))
+      (m_textStyle == TS_SUBSUBSECTION) ||
+      (m_textStyle == TS_HEADING5) || 
+      (m_textStyle == TS_HEADING6))
   {
     // Titles have a fixed font size 
     m_fontSize = configuration->GetFontSize(m_textStyle);
@@ -581,7 +599,11 @@ void TextCell::SetFont(int fontsize)
   }
 
   wxASSERT(Scale_Px(m_fontSize) > 0);
+#if wxCHECK_VERSION(3, 1, 2)
+  font.SetFractionalPointSize(Scale_Px(m_fontSize));
+#else
   font.SetPointSize(Scale_Px(m_fontSize));
+#endif
 
   wxASSERT_MSG(font.IsOk(),
                _("Seems like something is broken with a font. Installing http://www.math.union.edu/~dpvc/jsmath/download/jsMath-fonts.html and checking \"Use JSmath fonts\" in the configuration dialogue should fix it."));
@@ -1118,6 +1140,7 @@ wxString TextCell::ToMathML()
   {
     case TS_GREEK_CONSTANT:
       text = GetGreekStringUnicode();
+      break;
     case TS_SPECIAL_CONSTANT:
     {
       // The "d" from d/dt can be written as a special unicode symbol. But firefox doesn't
