@@ -281,6 +281,8 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
   textStyle.Add(_("Section"));
   textStyle.Add(_("Subsection"));
   textStyle.Add(_("Subsubsection"));
+  textStyle.Add(_("Heading5"));
+  textStyle.Add(_("Heading6"));
   m_textStyle = new wxChoice(this, tb_changeStyle, wxDefaultPosition, wxDefaultSize, textStyle);
   m_textStyle->SetToolTip(_("For faster creation of cells the following shortcuts exist:\n\n"
                             "   Ctrl+0: Math cell\n"
@@ -288,7 +290,10 @@ ToolBar::ToolBar(wxWindow *parent) : wxAuiToolBar(parent,-1, wxDefaultPosition,
                             "   Ctrl+2: Title cell\n"
                             "   Ctrl+3: Section cell\n"
                             "   Ctrl+4: Subsection cell\n"
-                            "   Ctrl+5: Sub-Subsection cell"));
+                            "   Ctrl+5: Sub-Subsection cell\n"
+                            "   Ctrl+6: Heading5 cell\n"
+                            "   Ctrl+7: Heading6 cell\n"
+                            ));
   AddControl(m_textStyle);
   
   // Seems like on MSW changing the image of this button has strange side-effects
@@ -361,6 +366,12 @@ void ToolBar::SetDefaultCellStyle()
   case 5:
     m_defaultCellStyle = GC_TYPE_SUBSUBSECTION;
     break;
+  case 6:
+    m_defaultCellStyle = GC_TYPE_HEADING5;
+    break;
+  case 7:
+    m_defaultCellStyle = GC_TYPE_HEADING6;
+    break;
   default:
   {}
   }
@@ -386,9 +397,15 @@ int ToolBar::GetCellStyle()
     return GC_TYPE_SUBSUBSECTION;
     break;
   case 6:
-    return GC_TYPE_IMAGE;
+    return GC_TYPE_HEADING5;
     break;
   case 7:
+    return GC_TYPE_HEADING6;
+    break;
+  case 8:
+    return GC_TYPE_IMAGE;
+    break;
+  case 9:
     return GC_TYPE_PAGEBREAK;
     break;
   default:
@@ -407,6 +424,8 @@ void ToolBar::SetCellStyle(int style)
   case GC_TYPE_SECTION:
   case GC_TYPE_SUBSECTION:
   case GC_TYPE_SUBSUBSECTION:
+  case GC_TYPE_HEADING5:
+  case GC_TYPE_HEADING6:
     break;
   default:
     style = m_defaultCellStyle;
@@ -432,6 +451,12 @@ void ToolBar::SetCellStyle(int style)
   case GC_TYPE_SUBSUBSECTION:
     m_textStyle->SetSelection(5);
     break;
+  case GC_TYPE_HEADING5:
+    m_textStyle->SetSelection(6);
+    break;
+  case GC_TYPE_HEADING6:
+    m_textStyle->SetSelection(7);
+    break;
   }
 }
 
@@ -478,7 +503,6 @@ void ToolBar::AnimationButtonState(AnimationStartStopState state)
 
 void ToolBar::OnSize(wxSizeEvent &event)
 {
-  wxLogMessage("ToolBar resizes");
   Refresh();
   event.Skip();
 }
