@@ -26,7 +26,7 @@
   This file defines the class wxMaximaFrame
 
   wxMaximaFrame is responsible for everything that is displayed around the actual
-  worksheet - which is displayed by MathCtrl and whose logic partially is defined in
+  worksheet - which is displayed by Worksheet and whose logic partially is defined in
   wxMaxima.
  */
 #include "wxMaximaFrame.h"
@@ -37,6 +37,7 @@
 #include <wx/image.h>
 #include <wx/filename.h>
 #include <wx/fileconf.h>
+#include <wx/persist/toplevel.h>
 #include "wxMaximaIcon.h"
 
 wxMaximaFrame::wxMaximaFrame(wxWindow *parent, int id, const wxString &title,
@@ -48,6 +49,22 @@ wxMaximaFrame::wxMaximaFrame(wxWindow *parent, int id, const wxString &title,
   m_unsavedDocuments(wxT("unsaved")),
   m_recentPackages(wxT("packages"))
 {
+  m_errorRedirector = NULL;
+  SetName(title);
+  if(!wxPersistenceManager::Get().RegisterAndRestore(this))
+  {
+    wxSize winSize = wxDefaultSize;
+    if(winSize == wxDefaultSize)
+    {
+      winSize = wxSize(wxSystemSettings::GetMetric ( wxSYS_SCREEN_X )*.75,
+                    wxSystemSettings::GetMetric ( wxSYS_SCREEN_Y )*.75);
+      if (winSize.x<800) winSize.x=800;
+      if (winSize.y<600) winSize.y=600;
+    }
+    SetSize(winSize);
+  }
+
+  m_logPanelTarget = NULL;
   m_isNamed = false;
   m_configFileName = configFile,
   m_updateEvaluationQueueLengthDisplay = true;
@@ -64,13 +81,13 @@ wxMaximaFrame::wxMaximaFrame(wxWindow *parent, int id, const wxString &title,
   wxDialog::EnableLayoutAdaptation(wxDIALOG_ADAPTATION_MODE_ENABLED);
 
   // console
-  m_console = new MathCtrl(this, -1, wxDefaultPosition, wxDefaultSize);
+  m_worksheet = new Worksheet(this, -1);
 
   // history
   m_history = new History(this, -1);
 
   // The table of contents
-  m_console->m_tableOfContents = new TableOfContents(this, -1, &m_console->m_configuration);
+  m_worksheet->m_tableOfContents = new TableOfContents(this, -1, &m_worksheet->m_configuration);
 
   m_xmlInspector = new XmlInspector(this, -1);
   SetupMenu();
@@ -84,20 +101,22 @@ wxMaximaFrame::wxMaximaFrame(wxWindow *parent, int id, const wxString &title,
   StatusMaximaBusy(waiting);
 
   // Add some shortcuts that aren't automatically set by menu entries.
-  wxAcceleratorEntry entries[11];
+  wxAcceleratorEntry entries[14];
   entries[0].Set(wxACCEL_CTRL, WXK_TAB, menu_autocomplete);
   entries[1].Set(wxACCEL_CTRL, WXK_SPACE, menu_autocomplete);
   entries[2].Set(wxACCEL_CTRL | wxACCEL_SHIFT, WXK_TAB, menu_autocomplete_templates);
   entries[3].Set(wxACCEL_CTRL | wxACCEL_SHIFT, WXK_SPACE, menu_autocomplete_templates);
-  entries[4].Set(wxACCEL_ALT, wxT('I'), MathCtrl::menu_zoom_in);
-  entries[5].Set(wxACCEL_ALT, wxT('O'), MathCtrl::menu_zoom_out);
+  entries[4].Set(wxACCEL_ALT, wxT('I'), Worksheet::menu_zoom_in);
+  entries[5].Set(wxACCEL_ALT, wxT('O'), Worksheet::menu_zoom_out);
   entries[6].Set(wxACCEL_CTRL | wxACCEL_SHIFT, WXK_ESCAPE, menu_convert_to_code);
-  entries[6].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('1'), menu_convert_to_comment);
-  entries[7].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('2'), menu_convert_to_title);
-  entries[8].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('3'), menu_convert_to_section);
-  entries[9].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('4'), menu_convert_to_subsection);
-  entries[10].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('5'), menu_convert_to_subsubsection);
-  wxAcceleratorTable accel(11, entries);
+  entries[7].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('1'), menu_convert_to_comment);
+  entries[8].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('2'), menu_convert_to_title);
+  entries[9].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('3'), menu_convert_to_section);
+  entries[10].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('4'), menu_convert_to_subsection);
+  entries[11].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('5'), menu_convert_to_subsubsection);
+  entries[12].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('6'), menu_convert_to_heading5);
+  entries[13].Set(wxACCEL_CTRL | wxACCEL_SHIFT, wxT('7'), menu_convert_to_heading6);
+  wxAcceleratorTable accel(14, entries);
   SetAcceleratorTable(accel);
 
   Move(pos);
@@ -107,6 +126,16 @@ wxMaximaFrame::wxMaximaFrame(wxWindow *parent, int id, const wxString &title,
   do_layout();
 }
 
+wxSize wxMaximaFrame::DoGetBestClientSize() const
+{
+  wxSize size(wxSystemSettings::GetMetric ( wxSYS_SCREEN_X )*.6,
+              wxSystemSettings::GetMetric ( wxSYS_SCREEN_Y )*.6);
+  if (size.x<800) size.x=800;
+  if (size.y<600) size.y=600;
+  return size;
+}
+
+
 void wxMaximaFrame::EvaluationQueueLength(int length, int numberOfCommands)
 {
   if ((length != m_EvaluationQueueLength) || (m_commandsLeftInCurrentCell != numberOfCommands))
@@ -127,42 +156,42 @@ void wxMaximaFrame::UpdateStatusMaximaBusy()
       switch (m_StatusMaximaBusy)
       {
         case process_wont_start:
-          m_newStatusText = _("Cannot start the maxima binary");
+          RightStatusText(_("Cannot start the maxima binary"));
           break;
         case userinput:
           m_MenuBar->Enable(menu_remove_output, false);
-          m_newStatusText = _("Maxima asks a question");
+          RightStatusText(_("Maxima asks a question"));
           break;
         case waiting:
-          m_console->m_cellPointers.SetWorkingGroup(NULL);
+          m_worksheet->m_cellPointers.SetWorkingGroup(NULL);
           // If we evaluated a cell that produces no output we still want the
           // cell to be unselected after evaluating it.
-          if (m_console->FollowEvaluation())
-            m_console->SetSelection(NULL);
+          if (m_worksheet->FollowEvaluation())
+            m_worksheet->SetSelection(NULL);
 
           m_MenuBar->Enable(menu_remove_output, true);
-          m_newStatusText = _("Ready for user input");
+          RightStatusText(_("Ready for user input"));
           // We don't evaluate any cell right now.
           break;
         case calculating:
           m_MenuBar->Enable(menu_remove_output, false);
-          m_newStatusText = _("Maxima is calculating");
+          RightStatusText(_("Maxima is calculating"), false);
           break;
         case transferring:
           m_MenuBar->Enable(menu_remove_output, false);
-          m_newStatusText = _("Reading Maxima output");
+          RightStatusText(_("Reading Maxima output"),false);
           break;
         case parsing:
           m_MenuBar->Enable(menu_remove_output, false);
-          m_newStatusText = ("Parsing output");
+          RightStatusText(_("Parsing output"),false);
           break;
         case disconnected:
           m_MenuBar->Enable(menu_remove_output, false);
-          m_newStatusText = _("Not connected to maxima");
+          RightStatusText(_("Not connected to maxima"));
           break;
         case wait_for_start:
           m_MenuBar->Enable(menu_remove_output, false);
-          m_newStatusText = _("Maxima started. Waiting for connection...");
+          RightStatusText(_("Maxima started. Waiting for connection..."));
           break;
       }
     }
@@ -174,7 +203,7 @@ void wxMaximaFrame::StatusSaveStart()
 {
   m_forceStatusbarUpdate = true;
   m_StatusSaving = true;
-  m_newStatusText = _("Saving...");
+  RightStatusText(_("Saving..."));
 }
 
 void wxMaximaFrame::StatusSaveFinished()
@@ -184,14 +213,14 @@ void wxMaximaFrame::StatusSaveFinished()
   if (m_StatusMaximaBusy != waiting)
     StatusMaximaBusy(m_StatusMaximaBusy);
   else
-    m_newStatusText = _("Saving successful.");
+    RightStatusText(_("Saving successful."));
 }
 
 void wxMaximaFrame::StatusExportStart()
 {
   m_forceStatusbarUpdate = true;
   m_StatusSaving = true;
-  m_newStatusText = _("Exporting...");
+  RightStatusText(_("Exporting..."));
 }
 
 void wxMaximaFrame::StatusExportFinished()
@@ -201,25 +230,29 @@ void wxMaximaFrame::StatusExportFinished()
   if (m_StatusMaximaBusy != waiting)
     StatusMaximaBusy(m_StatusMaximaBusy);
   else
-    m_newStatusText = _("Export successful.");
+    RightStatusText(_("Export successful."));
 }
 
 void wxMaximaFrame::StatusSaveFailed()
 {
   m_forceStatusbarUpdate = true;
   m_StatusSaving = false;
-  m_newStatusText = _("Saving failed.");
+  RightStatusText(_("Saving failed."));
 }
 
 void wxMaximaFrame::StatusExportFailed()
 {
   m_forceStatusbarUpdate = true;
   m_StatusSaving = false;
-  m_newStatusText = _("Export failed.");
+  RightStatusText(_("Export failed."));
 }
 
 wxMaximaFrame::~wxMaximaFrame()
 {
+  wxDELETE(m_errorRedirector);
+  // We are about to delete the panel all log messages are output to =>
+  // create a new log target.
+  wxLog::SetActiveTarget(new wxLogGui());
   wxString perspective = m_manager.SavePerspective();
 
   wxConfig::Get()->Write(wxT("AUI/perspective"), perspective);
@@ -229,9 +262,9 @@ wxMaximaFrame::~wxMaximaFrame()
   // or similar events pending for the wxWindows we want to free the memory
   // for.
   m_history->Destroy();
-  m_console->m_tableOfContents->Destroy();
-  m_console->m_tableOfContents = NULL;
-  m_console->Destroy();
+  m_worksheet->m_tableOfContents->Destroy();
+  m_worksheet->m_tableOfContents = NULL;
+  m_worksheet->Destroy();
 }
 
 void wxMaximaFrame::set_properties()
@@ -254,13 +287,13 @@ void wxMaximaFrame::set_properties()
   SetTitle(_("untitled"));
 #endif
 
-  m_console->SetBackgroundColour(wxColour(wxT("WHITE")));
-  m_console->SetMinSize(wxSize(100, 100));
+  m_worksheet->SetBackgroundColour(wxColour(wxT("WHITE")));
+  m_worksheet->SetMinSize(wxSize(100, 100));
 }
 
 void wxMaximaFrame::do_layout()
 {
-  m_manager.AddPane(m_console,
+  m_manager.AddPane(m_worksheet,
                     wxAuiPaneInfo().Name(wxT("console")).
                             Center().
                             CloseButton(false).
@@ -278,7 +311,7 @@ void wxMaximaFrame::do_layout()
                             PaneBorder(true).
                             Right());
 
-  m_manager.AddPane(m_console->m_tableOfContents,
+  m_manager.AddPane(m_worksheet->m_tableOfContents,
                     wxAuiPaneInfo().Name(wxT("structure")).
                             Show(true).CloseButton().PinButton().
                             TopDockable(true).
@@ -386,15 +419,15 @@ void wxMaximaFrame::do_layout()
                     PaneBorder(true).
                     Left());
 
-  m_console->m_mainToolBar = new ToolBar(this);
+  m_worksheet->m_mainToolBar = new ToolBar(this);
   
-  m_manager.AddPane(m_console->m_mainToolBar,
+  m_manager.AddPane(m_worksheet->m_mainToolBar,
                     wxAuiPaneInfo().Name(wxT("toolbar")).
                     ToolbarPane().Top().Resizable(true).
                     TopDockable(true).Show(true).
                     BottomDockable(true).
                     LeftDockable(false).
-                    RightDockable(false).Gripper(false)
+                    RightDockable(false).Gripper(false).Row(1).Fixed()
     );
 
   m_manager.GetPane(wxT("greek")) = m_manager.GetPane(wxT("greek")).
@@ -519,15 +552,15 @@ void wxMaximaFrame::SetupMenu()
   m_EditMenu->Append(menu_cut, _("Cut\tCtrl+X"),
                      _("Cut selection"),
                      wxITEM_NORMAL);
-  APPEND_MENU_ITEM(m_EditMenu, menu_copy_from_console, _("&Copy\tCtrl+C"),
+  APPEND_MENU_ITEM(m_EditMenu, menu_copy_from_worksheet, _("&Copy\tCtrl+C"),
                    _("Copy selection"), wxT("gtk-copy"));
-  m_EditMenu->Append(menu_copy_text_from_console, _("Copy as Text\tCtrl+Shift+C"),
+  m_EditMenu->Append(menu_copy_text_from_worksheet, _("Copy as Text\tCtrl+Shift+C"),
                      _("Copy selection from document as text"),
                      wxITEM_NORMAL);
-  m_EditMenu->Append(menu_copy_tex_from_console, _("Copy as LaTeX"),
+  m_EditMenu->Append(menu_copy_tex_from_worksheet, _("Copy as LaTeX"),
                      _("Copy selection from document in LaTeX format"),
                      wxITEM_NORMAL);
-  m_EditMenu->Append(MathCtrl::popid_copy_mathml, _("Copy as MathML"),
+  m_EditMenu->Append(Worksheet::popid_copy_mathml, _("Copy as MathML"),
                      _("Copy selection from document in a MathML format many word processors can display as 2d equation"),
                      wxITEM_NORMAL);
   m_EditMenu->Append(menu_copy_as_bitmap, _("Copy as Image"),
@@ -558,7 +591,7 @@ void wxMaximaFrame::SetupMenu()
                      _("Save selection from document to an image file"),
                      wxITEM_NORMAL);
   m_EditMenu->AppendSeparator();
-  m_EditMenu->Append(MathCtrl::popid_comment_selection, _("Comment selection\tCtrl+/"),
+  m_EditMenu->Append(Worksheet::popid_comment_selection, _("Comment selection\tCtrl+/"),
                      _("Comment out the currently selected text"),
                      wxITEM_NORMAL);
   m_EditMenu->AppendSeparator();
@@ -584,7 +617,7 @@ void wxMaximaFrame::SetupMenu()
   m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_format, _("Insert Cell\tAlt+Shift+C"));
   m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_draw, _("Plot using Draw"));
   m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_log,   _("Debug messages"));
-  m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_xmlInspector, _("XML Inspector"));
+  m_Maxima_Panes_Sub->AppendCheckItem(menu_pane_xmlInspector, _("Raw XML Monitor"));
   m_Maxima_Panes_Sub->AppendSeparator();
   m_Maxima_Panes_Sub->AppendCheckItem(ToolBar::tb_hideCode, _("Hide Code Cells\tAlt+Ctrl+H"));
   m_Maxima_Panes_Sub->Append(menu_pane_hideall, _("Hide All Toolbars\tAlt+Shift+-"), _("Hide all panes"),
@@ -611,9 +644,9 @@ void wxMaximaFrame::SetupMenu()
 
 
   m_Maxima_Panes_Sub->AppendSeparator();
-  APPEND_MENU_ITEM(m_Maxima_Panes_Sub, MathCtrl::menu_zoom_in, _("Zoom &In\tCtrl++"),
+  APPEND_MENU_ITEM(m_Maxima_Panes_Sub, Worksheet::menu_zoom_in, _("Zoom &In\tCtrl++"),
                    _("Zoom in 10%"), wxT("gtk-zoom-in"));
-  APPEND_MENU_ITEM(m_Maxima_Panes_Sub, MathCtrl::menu_zoom_out, _("Zoom Ou&t\tCtrl+-"),
+  APPEND_MENU_ITEM(m_Maxima_Panes_Sub, Worksheet::menu_zoom_out, _("Zoom Ou&t\tCtrl+-"),
                    _("Zoom out 10%"), wxT("gtk-zoom-out"));
   // zoom submenu
   m_Edit_Zoom_Sub = new wxMenu;
@@ -669,6 +702,10 @@ void wxMaximaFrame::SetupMenu()
                      _("Insert a new subsection cell"));
   m_CellMenu->Append(menu_add_subsubsection, _("Insert S&ubsubsection Cell\tCtrl+5"),
                      _("Insert a new subsubsection cell"));
+  m_CellMenu->Append(menu_add_heading5, _("Insert heading5 Cell\tCtrl+6"),
+                     _("Insert a new heading5 cell"));
+  m_CellMenu->Append(menu_add_heading6, _("Insert heading6 Cell\tCtrl+7"),
+                     _("Insert a new heading7 cell"));
   m_CellMenu->Append(menu_add_pagebreak, _("Insert Page Break"),
                      _("Insert a page break"));
   m_CellMenu->Append(menu_insert_image, _("Insert Image..."),
@@ -684,12 +721,12 @@ void wxMaximaFrame::SetupMenu()
   m_CellMenu->Append(menu_history_next, _("Next Command\tAlt+Down"),
                      _("Recall next command from history"), wxITEM_NORMAL);
   m_CellMenu->AppendSeparator();
-  m_CellMenu->Append(MathCtrl::popid_merge_cells, _("Merge Cells\tCtrl+M"),
+  m_CellMenu->Append(Worksheet::popid_merge_cells, _("Merge Cells\tCtrl+M"),
                      _("Merge the text from two input cells into one"), wxITEM_NORMAL);
-  m_CellMenu->Append(MathCtrl::popid_divide_cell, _("Divide Cell\tCtrl+D"),
+  m_CellMenu->Append(Worksheet::popid_divide_cell, _("Divide Cell\tCtrl+D"),
                      _("Divide this input cell into two cells"), wxITEM_NORMAL);
   m_CellMenu->AppendSeparator();
-  m_CellMenu->AppendCheckItem(MathCtrl::popid_auto_answer, _("Automatically answer questions"),
+  m_CellMenu->AppendCheckItem(Worksheet::popid_auto_answer, _("Automatically answer questions"),
                      _("Automatically fill in answers known from the last run"));
 
   m_MenuBar->Append(m_CellMenu, _("Ce&ll"));
@@ -1300,7 +1337,7 @@ void wxMaximaFrame::RemoveTempAutosavefile()
   {
     // Don't delete the file if we have opened it and haven't saved it under a
     // different name yet.
-    if(wxFileExists(m_tempfileName) && (m_tempfileName != m_console->m_currentFile))
+    if(wxFileExists(m_tempfileName) && (m_tempfileName != m_worksheet->m_currentFile))
       wxRemoveFile(m_tempfileName);
   }
   m_tempfileName = wxEmptyString;
@@ -1362,7 +1399,7 @@ void wxMaximaFrame::ShowPane(Event id, bool show)
       break;
     case menu_pane_structure:
       m_manager.GetPane(wxT("structure")).Show(show);
-      m_console->m_tableOfContents->UpdateTableOfContents(m_console->GetTree(), m_console->GetHCaret());
+      m_worksheet->m_tableOfContents->UpdateTableOfContents(m_worksheet->GetTree(), m_worksheet->GetHCaret());
       break;
     case menu_pane_xmlInspector:
       m_manager.GetPane(wxT("XmlInspector")).Show(show);
@@ -1531,7 +1568,7 @@ void wxMaximaFrame::CharacterButtonPressed(wxMouseEvent &event)
 {
   wxChar ch = event.GetId();
   wxString ch_string(ch);
-  m_console->InsertText(ch_string);
+  m_worksheet->InsertText(ch_string);
 }
 
 wxPanel *wxMaximaFrame::CharButton(wxPanel *parent, wxChar ch, wxString description, bool WXUNUSED(matchesMaximaCommand))
@@ -1637,7 +1674,13 @@ wxPanel *wxMaximaFrame::CreateLogPane()
   vbox->Add(textCtrl, wxSizerFlags().Expand().Proportion(10));
 
   panel->SetSizerAndFit(vbox);
-  wxLog::SetActiveTarget(new wxLogTextCtrl(textCtrl));
+  wxLog::SetActiveTarget(m_logPanelTarget = new wxLogTextCtrl(textCtrl));
+  m_errorRedirector = new ErrorRedirector(new wxLogGui());
+
+  // m_logPanelTarget->SetRepetitionCounting();
+  // m_logPanelTarget->DisableTimestamp();
+  SetMinSize(wxSize(wxSystemSettings::GetMetric ( wxSYS_SCREEN_X )/10,
+                    wxSystemSettings::GetMetric ( wxSYS_SCREEN_Y )/10));
   return panel;
 }
 
@@ -1761,11 +1804,15 @@ wxPanel *wxMaximaFrame::CreateFormatPane()
                          wxDefaultPosition, wxDefaultSize, wxBORDER_NONE | wxBU_EXACTFIT), 0, style, border);
   grid->Add(new wxButton(panel, menu_format_title, _("Title"),
                          wxDefaultPosition, wxDefaultSize, wxBORDER_NONE | wxBU_EXACTFIT), 0, style, border);
+  grid->Add(new wxButton(panel, menu_format_section, _("Section"),
+                         wxDefaultPosition, wxDefaultSize, wxBORDER_NONE | wxBU_EXACTFIT), 0, style, border);
   grid->Add(new wxButton(panel, menu_format_subsection, _("Subsection"),
                          wxDefaultPosition, wxDefaultSize, wxBORDER_NONE | wxBU_EXACTFIT), 0, style, border);
   grid->Add(new wxButton(panel, menu_format_subsubsection, _("Subsubsection"),
                          wxDefaultPosition, wxDefaultSize, wxBORDER_NONE | wxBU_EXACTFIT), 0, style, border);
-  grid->Add(new wxButton(panel, menu_format_section, _("Section"),
+  grid->Add(new wxButton(panel, menu_format_heading5, _("Heading 5"),
+                         wxDefaultPosition, wxDefaultSize, wxBORDER_NONE | wxBU_EXACTFIT), 0, style, border);
+  grid->Add(new wxButton(panel, menu_format_heading6, _("Heading 6"),
                          wxDefaultPosition, wxDefaultSize, wxBORDER_NONE | wxBU_EXACTFIT), 0, style, border);
   grid->Add(new wxButton(panel, menu_format_image, _("Image"),
                          wxDefaultPosition, wxDefaultSize, wxBORDER_NONE | wxBU_EXACTFIT), 0, style, border);
